using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace PB
{
	public class PelletBehaviour : MonoBehaviour
	{
		public float fire_timer = 0.05f;
		public bool fired = false;

		private void Update()
		{
			if(fire_timer >= 0 || fired) {
				fire_timer -= Time.deltaTime;
				return;
			}

			fired = true;

			List<GameObject> bullets = new List<GameObject>();
			for(int i = 0; i < 8; i++) {
				GameObject bullet = ModAPI.CreatePhysicalObject("pbs12 pellet", Cache.sprites["bs12b"]);
				bullet.transform.position = transform.position;
				bullet.transform.rotation = transform.rotation;

				bullet.GetComponent<Rigidbody2D>().velocity = base.GetComponent<Rigidbody2D>().velocity;
				bullet.GetComponent<Rigidbody2D>().AddForce(new Vector2(UnityEngine.Random.Range(0, 100), UnityEngine.Random.Range(0, 100)));

				IgnoreCollisionStackController.RequestIgnoreCollision(bullet.GetComponent<Collider2D>(), base.GetComponent<Collider2D>());
				bullets.Add(bullet);
			}

			for(int i = 0; i < 7; i++) {
				Collider2D coll = bullets[i].GetComponent<Collider2D>();

				for(int j = 0; j < 8; j++) {
					if(j == i) continue;
					IgnoreCollisionStackController.RequestIgnoreCollision(coll, bullets[j].GetComponent<Collider2D>());
				}
			}

			base.GetComponent<Rigidbody2D>().velocity *= 0.25f;
		}
	}

	public class PrimerBehaviour : MonoBehaviour { }

	public class BulletBehaviour : MonoBehaviour
	{
		public string type;

		public bool fired = false;

		public float recoil_force = 10000;

		public Vector2[] coll_path;

		private void Start()
		{
			if(fired)
				base.GetComponent<SpriteRenderer>().sprite = Cache.sprites[type + "f"];
			else
				base.GetComponent<SpriteRenderer>().sprite = Cache.sprites[type];

			if(!fired && coll_path != null) {
				UnityEngine.Object.Destroy(base.GetComponent<Collider2D>());

				PolygonCollider2D coll = gameObject.AddComponent<PolygonCollider2D>();

				coll.SetPath(0, coll_path);
			} else {
				gameObject.FixColliders();
			}

			PhysicalBehaviour phys = base.GetComponent<PhysicalBehaviour>();

			if(!fired) {
				phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("setBulletKickback", "Set recoil force", "Set bullet recoil force", delegate
				{
					Utils.OpenFloatInputDialog(recoil_force, this, delegate(BulletBehaviour t, float v)
					{
						if(v < 0)
							v = 0;
						t.recoil_force = v;
					}, "Set recoil force", "Target recoil force");
				}));
			}

			phys.RefreshOutline();

			PhysicsMaterial2D pm2d = UnityEngine.Object.Instantiate(phys.Properties.PhysicMaterial);

			//pm2d.friction = 0f;
			//pm2d.bounciness = 0.8f;

			phys.Properties.PhysicMaterial = pm2d;

			phys.Resizable = false;
		}

		private void OnCollisionEnter2D(Collision2D coll)
		{
			if((coll.gameObject.GetComponent<PrimerBehaviour>() == null && coll.gameObject.GetComponent<PhysicalBehaviour>().Charge <= 2) || fired)
				return;

			Vector3 right = transform.localRotation * Vector3.Scale(Vector3.right, transform.localScale);

			fired = true;

			float old_scale = Cache.sprites[type].bounds.size.x;

			base.GetComponent<SpriteRenderer>().sprite = Cache.sprites[type + "f"];

			if(type[1] != 's')
				gameObject.FixColliders();
			base.GetComponent<PhysicalBehaviour>().RefreshOutline();
			base.transform.position -= Vector3.Scale(right, new Vector3((old_scale - Cache.sprites[type + "f"].bounds.size.x) / 2, 0, 0));

			GameObject bullet = ModAPI.CreatePhysicalObject("p" + type, Cache.sprites[type + "a"]);
			bullet.transform.position = transform.position + right * 0.75f;
			bullet.transform.rotation = transform.rotation;

			if(transform.localScale.x < 0)
				bullet.transform.localScale = new Vector3(bullet.transform.localScale.x * -1, bullet.transform.localScale.y, bullet.transform.localScale.z);

			bullet.GetComponent<BoxCollider2D>().size -= new Vector2(3 * ModAPI.PixelSize, 5 * ModAPI.PixelSize);
			bullet.GetComponent<Rigidbody2D>().AddForce(right * 30000);
			base.GetComponent<Rigidbody2D>().AddForce(right * -recoil_force);

			ExplosionCreator.Explode(transform.position + right * 0.75f, 0);

			if(type[1] == 's')
				bullet.AddComponent<PelletBehaviour>();
		}
	};
}
