using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace PB
{
	public class Cache
	{
		public static Dictionary<string, Sprite> sprites = new Dictionary<string, Sprite>();
		public static Dictionary<string, AudioClip> sounds = new Dictionary<string, AudioClip>();
	}

	public class Mod
	{
		public static void Main()
		{
			CacheTextures();
			CacheSounds();

			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "PBullet 12px",
					DescriptionOverride = "12px bullet",
					CategoryOverride = ModAPI.FindCategory("Misc."),
					ThumbnailOverride = Cache.sprites["b12t"],
					AfterSpawn = (obj) => {
						BulletBehaviour bull = obj.GetOrAddComponent<BulletBehaviour>();
						bull.type = "b12";
						bull.coll_path = new Vector2[8];

						bull.coll_path[0] = new Vector2(-20.5f * ModAPI.PixelSize, 6 * ModAPI.PixelSize);
						bull.coll_path[1] = new Vector2(8.5f * ModAPI.PixelSize, 6 * ModAPI.PixelSize);
						bull.coll_path[2] = new Vector2(8.5f * ModAPI.PixelSize, 4 * ModAPI.PixelSize);
						bull.coll_path[3] = new Vector2(20.5f * ModAPI.PixelSize, 4 * ModAPI.PixelSize);
						bull.coll_path[4] = new Vector2(20.5f * ModAPI.PixelSize, -4 * ModAPI.PixelSize);
						bull.coll_path[5] = new Vector2(8.5f * ModAPI.PixelSize, -4 * ModAPI.PixelSize);
						bull.coll_path[6] = new Vector2(8.5f * ModAPI.PixelSize, -6 * ModAPI.PixelSize);
						bull.coll_path[7] = new Vector2(-20.5f * ModAPI.PixelSize, -6 * ModAPI.PixelSize);
					}
				}
			);

			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "PBullet 8px",
					DescriptionOverride = "8px bullet",
					CategoryOverride = ModAPI.FindCategory("Misc."),
					ThumbnailOverride = Cache.sprites["b8t"],
					AfterSpawn = (obj) => {
						BulletBehaviour bull = obj.GetOrAddComponent<BulletBehaviour>();
						bull.type = "b8";

						bull.coll_path = new Vector2[8];

						bull.coll_path[0] = new Vector2(-11.5f * ModAPI.PixelSize, 4 * ModAPI.PixelSize);
						bull.coll_path[1] = new Vector2(5.5f * ModAPI.PixelSize, 4 * ModAPI.PixelSize);
						bull.coll_path[2] = new Vector2(5.5f * ModAPI.PixelSize, 2 * ModAPI.PixelSize);
						bull.coll_path[3] = new Vector2(11.5f * ModAPI.PixelSize, 2 * ModAPI.PixelSize);
						bull.coll_path[4] = new Vector2(11.5f * ModAPI.PixelSize, -2 * ModAPI.PixelSize);
						bull.coll_path[5] = new Vector2(5.5f * ModAPI.PixelSize, -2 * ModAPI.PixelSize);
						bull.coll_path[6] = new Vector2(5.5f * ModAPI.PixelSize, -4 * ModAPI.PixelSize);
						bull.coll_path[7] = new Vector2(-11.5f * ModAPI.PixelSize, -4 * ModAPI.PixelSize);
					}
				}
			);

			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "PBullet 14px",
					DescriptionOverride = "14px bullet",
					CategoryOverride = ModAPI.FindCategory("Misc."),
					ThumbnailOverride = Cache.sprites["b14t"],
					AfterSpawn = (obj) => {
						BulletBehaviour bull = obj.GetOrAddComponent<BulletBehaviour>();
						bull.type = "b14";

						bull.coll_path = new Vector2[8];

						bull.coll_path[0] = new Vector2(-32f * ModAPI.PixelSize, 7 * ModAPI.PixelSize);
						bull.coll_path[1] = new Vector2(15.5f * ModAPI.PixelSize, 7 * ModAPI.PixelSize);
						bull.coll_path[2] = new Vector2(15.5f * ModAPI.PixelSize, 5 * ModAPI.PixelSize);
						bull.coll_path[3] = new Vector2(32f * ModAPI.PixelSize, 4 * ModAPI.PixelSize);
						bull.coll_path[4] = new Vector2(32f * ModAPI.PixelSize, -4 * ModAPI.PixelSize);
						bull.coll_path[5] = new Vector2(15.5f * ModAPI.PixelSize, -5 * ModAPI.PixelSize);
						bull.coll_path[6] = new Vector2(15.5f * ModAPI.PixelSize, -7 * ModAPI.PixelSize);
						bull.coll_path[7] = new Vector2(-32f * ModAPI.PixelSize, -7 * ModAPI.PixelSize);
					}
				}
			);

			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "PBullet S12px",
					DescriptionOverride = "S12px bullet",
					CategoryOverride = ModAPI.FindCategory("Misc."),
					ThumbnailOverride = Cache.sprites["bs12t"],
					AfterSpawn = (obj) => {
						BulletBehaviour bull = obj.GetOrAddComponent<BulletBehaviour>();
						bull.type = "bs12";

						bull.coll_path = new Vector2[8];

						bull.coll_path[0] = new Vector2(-21f * ModAPI.PixelSize, 7.5f * ModAPI.PixelSize);
						bull.coll_path[1] = new Vector2(-19.5f * ModAPI.PixelSize, 7.5f * ModAPI.PixelSize);
						bull.coll_path[2] = new Vector2(-19.5f * ModAPI.PixelSize, 6f * ModAPI.PixelSize);
						bull.coll_path[3] = new Vector2(21f * ModAPI.PixelSize, 6f * ModAPI.PixelSize);
						bull.coll_path[4] = new Vector2(21f * ModAPI.PixelSize, -6f * ModAPI.PixelSize);
						bull.coll_path[5] = new Vector2(-19.5f * ModAPI.PixelSize, -6f * ModAPI.PixelSize);
						bull.coll_path[6] = new Vector2(-19.5f * ModAPI.PixelSize, -7.5f * ModAPI.PixelSize);
						bull.coll_path[7] = new Vector2(-21f * ModAPI.PixelSize, -7.5f * ModAPI.PixelSize);
					}
				}
			);

			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameOverride = "PBullet Primer",
					DescriptionOverride = "Primes a bullet",
					CategoryOverride = ModAPI.FindCategory("Misc."),
					ThumbnailOverride = Cache.sprites["primert"],
					AfterSpawn = (obj) => {
						obj.GetOrAddComponent<PrimerBehaviour>();
						obj.GetComponent<SpriteRenderer>().sprite = Cache.sprites["primer"];
						obj.FixColliders();
					}
				}
			);
		}

		public static void CacheTextures()
		{
			if(Cache.sprites.Count > 0) return;

			Cache.sprites.Add("b8", ModAPI.LoadSprite("b8px.png"));
			Cache.sprites.Add("b8a", ModAPI.LoadSprite("b8pxa.png"));
			Cache.sprites.Add("b8f", ModAPI.LoadSprite("b8pxf.png"));
			Cache.sprites.Add("b8t", ModAPI.LoadSprite("b8pxt.png"));
			Cache.sprites.Add("b12", ModAPI.LoadSprite("b12px.png"));
			Cache.sprites.Add("b12a", ModAPI.LoadSprite("b12pxa.png"));
			Cache.sprites.Add("b12f", ModAPI.LoadSprite("b12pxf.png"));
			Cache.sprites.Add("b12t", ModAPI.LoadSprite("b12pxt.png"));
			Cache.sprites.Add("b14", ModAPI.LoadSprite("b14px.png"));
			Cache.sprites.Add("b14a", ModAPI.LoadSprite("b14pxa.png"));
			Cache.sprites.Add("b14f", ModAPI.LoadSprite("b14pxf.png"));
			Cache.sprites.Add("b14t", ModAPI.LoadSprite("b14pxt.png"));
			Cache.sprites.Add("bs12", ModAPI.LoadSprite("bs12px.png"));
			Cache.sprites.Add("bs12a", ModAPI.LoadSprite("bs12pxa.png"));
			Cache.sprites.Add("bs12f", ModAPI.LoadSprite("bs12pxf.png"));
			Cache.sprites.Add("bs12b", ModAPI.LoadSprite("bs12pxb.png"));
			Cache.sprites.Add("bs12t", ModAPI.LoadSprite("bs12pxt.png"));
			Cache.sprites.Add("primer", ModAPI.LoadSprite("primer.png"));
			Cache.sprites.Add("primert", ModAPI.LoadSprite("primert.png"));
			Cache.sprites.Add("silencer", ModAPI.LoadSprite("silencer.png"));
		}

		public static void CacheSounds()
		{
			if(Cache.sounds.Count > 0) return;
		}
	}
}
